/*BEGIN OCO COPYRIGHT
 *************************************************************************
 *
 * IBM Confidential
 * OCO Source Materials
 * 5724-L01, 5655-N53, 5724-I82, 5655-R15
 * (C) Copyright IBM Corporation 2006.
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *
 **************************************************************************
 *END OCO COPYRIGHT
 */
 
/*
 * Created on Jun 16, 2006
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 *
 */
package com.ibm.websphere.samples.databinding;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.logging.Logger;

import javax.resource.cci.Record;

import com.ibm.j2ca.base.UnstructuredRecord;
import com.ibm.websphere.samples.databinding.exceptions.NameValueBindingException;
import com.ibm.websphere.samples.databinding.exceptions.NameValueHandlerException;
import com.ibm.websphere.samples.databinding.log.BindingLogUtil;
import com.ibm.websphere.samples.databinding.util.NameValueDataHandler;
import commonj.connector.runtime.DataBindingException;
import commonj.connector.runtime.RecordHolderDataBinding;
import commonj.sdo.DataObject;


/**
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 *
 * @author zlainwala
 */
public class NameValueEISDataBinding
    implements RecordHolderDataBinding
{
	//	BEGIN COPYRIGHT VARIABLE
	public static final String COPYRIGHT = "(C) Copyright IBM Corporation 2006.";	//$NON-NLS-$1
	// END COPYRIGHT VARIABLE
	
    private static final long serialVersionUID = 5201599634721924280L;
    private UnstructuredRecord record;
    
    private static final String CLASSNAME = NameValueEISDataBinding.class.getName();
    private static Logger LOGGER = BindingLogUtil.initLogger(CLASSNAME);
    private static final String CONTENT_TYPE = "text/namevalue";  //$NON-NLS-2$  //$NON-NLS-1$
    private static final String TARGETNAMESPACE = "TargetNamespace"; //$NON-NLS-2$  //$NON-NLS-1$ 
    
    /**
     * @see commonj.connector.runtime.RecordHolderDataBinding#getRecord()
     * @author zlainwala
     */
    public Record getRecord()
        throws DataBindingException
    {
        return record;
    }


    /**
     * @see commonj.connector.runtime.RecordHolderDataBinding#setRecord(javax.resource.cci.Record)
     * @author zlainwala
     */
    public void setRecord(Record unstructuredRecord)
        throws DataBindingException
    {
        record = (UnstructuredRecord) unstructuredRecord;
    }


    /**
     * @see commonj.connector.runtime.DataBinding#getDataObject()
     * @author zlainwala
     */
    public DataObject getDataObject()
    throws DataBindingException
    {
        BindingLogUtil.logEntry(LOGGER, CLASSNAME, "Entered method getDataObject()");  //$NON-NLS-1$
        try{
        	DataObject dataBindingConfigDataObject = EISDataBindingHelper.getBindingDataObject(record);
            HashMap bindingProperties = getBindingProperties(dataBindingConfigDataObject);    	            
            NameValueDataHandler nameValueDataHandler = new NameValueDataHandler();
            boolean isText = record.isText();
            if(isText)
            {
                String nameValueString = record.getText();
                DataObject dataObject = nameValueDataHandler.getDataObject(nameValueString, bindingProperties);
                return dataObject;
            }
            byte[] bytes = record.getBytes();
            DataObject dataObject = nameValueDataHandler.getDataObject(bytes, null);
            BindingLogUtil.logEntry(LOGGER, CLASSNAME, "Leaving method getDataObject()");  //$NON-NLS-1$
            return dataObject; 
        }
        catch(UnsupportedEncodingException e)
        {
            throw new DataBindingException(e);
        }catch(NameValueHandlerException e)
		{
    		throw new DataBindingException(e);
		}  
    }
    
    /**
     * @see commonj.connector.runtime.DataBinding#setDataObject(commonj.sdo.DataObject)
     * @author zlainwala
     */
    public void setDataObject(DataObject dataObject)
        throws DataBindingException
    {       
        BindingLogUtil.logEntry(LOGGER, CLASSNAME, "Entered method setDataObject(DataObject)");  //$NON-NLS-1$
        try{
	        DataObject dataBindingConfigDataObject = EISDataBindingHelper.getBindingDataObject(dataObject, CONTENT_TYPE);
	        HashMap bindingProperties = getBindingProperties(dataBindingConfigDataObject);    	
	        NameValueDataHandler nameValueDataHandler = new NameValueDataHandler();
	        byte[] nameValueBytes = nameValueDataHandler.getBytesFromDataObject(dataObject, bindingProperties);        
	        record = new UnstructuredRecord();
	        record.setBytes(nameValueBytes);
	        record.setContentType(CONTENT_TYPE);
        }
        catch(NameValueHandlerException e)
		{
    		throw new DataBindingException(e);
		}  
        BindingLogUtil.logEntry(LOGGER, CLASSNAME, "Leaving method setDataObject(DataObject)");  //$NON-NLS-1$
    }
    
    protected HashMap getBindingProperties(DataObject dataBindingConfigBO)
    	throws NameValueBindingException
    {
    	BindingLogUtil.logEntry(LOGGER, CLASSNAME, "Entering method getBindingProperties(DataObject)");  //$NON-NLS-1$ 
        
    	HashMap bindingProperties = new HashMap();   
    	try{
	        if(dataBindingConfigBO.isSet(TARGETNAMESPACE))
	        {
	        	String targetNamespace = dataBindingConfigBO.getString(TARGETNAMESPACE);	        	
	        	bindingProperties.put(TARGETNAMESPACE, targetNamespace);
	        }
    	}catch(Exception e)
		{
    		String msg = "Targetnamespace property is not set" + e.getMessage();
    		throw new NameValueBindingException(msg, e);
		}
        
        BindingLogUtil.logEntry(LOGGER, CLASSNAME, "Leaving method getBindingProperties(DataObject)");  //$NON-NLS-1$
        return bindingProperties;
    }
}
