/*BEGIN OCO COPYRIGHT
 *************************************************************************
 *
 * IBM Confidential
 * OCO Source Materials
 * 5724-L01, 5655-N53, 5724-I82, 5655-R15
 * (C) Copyright IBM Corporation 2006.
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *
 **************************************************************************
 *END OCO COPYRIGHT
 */

/**
 * The FixedWidthDataHandler converts a fixed width stream of data to a 
 * data object and vice versa. 
 * A fixedwidth stream with pad character ~ for a customer business object which 
 * contain properties
 * id, firstName, lastName, contained object address with id looks as 
 * follows Customer~~,10~~,John~~~~,Doe~~~~~,address~~~,101~.
 * There are configuration properties like padCharacater, unset, blank,
 * alignment, truncate, dataObjectNameSize, dataObjectCountSize,
 * encoding, fieldWidth, targetNamespace that can be
 * passed to the handler.
 * <p>
 * @ibm-api
 */
package com.ibm.websphere.samples.databinding.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.sdo.EProperty;
import org.eclipse.emf.ecore.sdo.EType;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.ExtendedMetaData;

import com.ibm.websphere.bo.BOFactory;
import com.ibm.websphere.samples.databinding.exceptions.FixedWidthHandlerException;
import com.ibm.websphere.samples.databinding.log.HandlerUtil;
import com.ibm.websphere.sca.ServiceManager;
import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;

/**
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 *
 * @author zlainwala
 */
public class FixedWidthDataHandler
{
    /** Automatically generated javadoc for: FIELDWIDTH */
    private static final int FIELDWIDTH = 15;
    /** Automatically generated javadoc for: DATAOBJECTNAMESIZE */
    private static final int DATAOBJECTNAMESIZE = 15;
    private final static String ALIGNMENTBOTH = "BOTH";  //$NON-NLS-1$
    private final static String ALIGNMENTLEFT = "LEFT";  //$NON-NLS-1$
    private final static String ALIGNMENTRIGHT = "RIGHT";  //$NON-NLS-1$

    //Properties on the handler
    private StringBuffer unset;
    private StringBuffer blank;
    private String alignment;
    private boolean truncate;
    private char padCharacter;
    private int dataObjectNameSize;
    private int dataObjectCountSize;
    private String encoding;
    private int fieldWidth = 0;
    private String targetNamespace;

    //for the BOString parsing
    private int startIndex; //of a line
    private int dataObjectStringLength;
    private String dataObjectString;

    private static ServiceManager serviceManager = new ServiceManager();
    private static final BOFactory BOFACTORY = (BOFactory) serviceManager
        .locateService("com/ibm/websphere/bo/BOFactory");  //$NON-NLS-1$

    private static final String CLASSNAME = DelimitedDataHandler.class.getName();
    private static final Logger LOGGER = HandlerUtil.initLogger(CLASSNAME);
    
    //	BEGIN COPYRIGHT VARIABLE
	public static final String COPYRIGHT = "(C) Copyright IBM Corporation 2006.";	//$NON-NLS-$1
	// END COPYRIGHT VARIABLE
    /**
     * The constructor initializes the variables
     */
    public FixedWidthDataHandler()
    {    	
        unset = new StringBuffer("unset");  //$NON-NLS-1$
        blank = new StringBuffer("");  //$NON-NLS-1$
        alignment = FixedWidthDataHandler.ALIGNMENTLEFT;
        truncate = true;
        padCharacter = '~';
        dataObjectNameSize = DATAOBJECTNAMESIZE;
        dataObjectCountSize = 4;
        fieldWidth = FIELDWIDTH;
        encoding = null;
        startIndex = 0; //of a line
        dataObjectStringLength = 0;
        dataObjectString = null;
        targetNamespace = new String("http://www.ibm.com/crm");  //$NON-NLS-1$
    }

    /**
     * Override the default configuration properties on the Handler based on the configuration
     * that is passed in.
     *
     * @param config 
     */
    protected void setupConfig(HashMap config)
    {
        if (config == null || config.isEmpty()) {
            return;
        }

        Iterator iterator = config.entrySet().iterator();
        while (iterator.hasNext()) {
            Entry entry = (Entry) iterator.next();
            if (entry.getKey().equals("unset")) {  //$NON-NLS-1$
                unset = new StringBuffer((String) entry.getValue());
                continue;
            }
            if (entry.getKey().equals("blank")) {  //$NON-NLS-1$
                blank = new StringBuffer((String) entry.getValue());
                continue;
            }
            if (entry.getKey().equals("encoding")) {  //$NON-NLS-1$
                encoding = (String) entry.getValue();
                continue;
            }
            if (entry.getKey().equals("alignment")) {  //$NON-NLS-1$
                alignment = (String) entry.getValue();
                continue;
            }
            if (entry.getKey().equals("truncate")) {  //$NON-NLS-1$
                truncate = ((Boolean) entry.getValue()).booleanValue();
                continue;
            }
            if (entry.getKey().equals("PadCharacter")) {  //$NON-NLS-1$
                padCharacter = ((String) entry.getValue()).charAt(0);
                continue;
            }
            if (entry.getKey().equals("dataObjectNameSize")) {  //$NON-NLS-1$
                dataObjectNameSize = ((Integer) entry.getValue()).intValue();
                continue;
            }
            if (entry.getKey().equals("dataObjectCountSize")) {  //$NON-NLS-1$
                dataObjectCountSize = ((Integer) entry.getValue()).intValue();
                continue;
            }
            if (entry.getKey().equals("FieldWidth")) {  //$NON-NLS-1$
                Integer integerValue = new Integer((String)entry.getValue());
                fieldWidth =  integerValue.intValue();
                continue;
            }
            if (entry.getKey().equals("TargetNamespace")) {  //$NON-NLS-1$
                targetNamespace = (String) entry.getValue();
                continue;
            }
        }
    }

    /**
     * Converts a fixedwidth byte stream to a data object. It uses a hashmap of 
     * configuration properties. The default values for these properties
     * are set in in the constructor.
     * If the config is null, the default values are used for parsing the
     * stream. The default values can be overridden by passing them in the
     * config hashmap.
     * 
     * @param bytes input bytes to be converted
     * @param config map of configuration properties
     * @return dataobject that corresponds to the byte stream
     * @throws FixedWidthHandlerException The bytes can be read
     */
    public DataObject getDataObject(byte[] bytes,
                                    HashMap config)
    	throws FixedWidthHandlerException
    {
        logEntry("getDataObject(byte[], HashMap");  //$NON-NLS-1$
        HandlerUtil.logConfig(LOGGER, CLASSNAME, "getDataObject(byte[], HashMap)", config);  //$NON-NLS-1$
        
        setupConfig(config);        
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try {
            outputStream.write(bytes);
        } catch (IOException e) {
            String message = "Error when reading the stream of input bytes";  //$NON-NLS-1$
            LOGGER.logp(Level.SEVERE, CLASSNAME, "getDataObject(byte[], HashMap)", message);  //$NON-NLS-1$
            throw new FixedWidthHandlerException(message, e);
        }
        
        dataObjectString = outputStream.toString();
        dataObjectStringLength = dataObjectString.length();

        LOGGER.logp(Level.FINEST, CLASSNAME, "getDataObject(byte[], HashMap)", //$NON-NLS-1$
                    "Incoming byte array is : \"{0}\"", dataObjectString);  //$NON-NLS-1$
        
        DataObject responseObj = getDataObjectFromString(null);

        HandlerUtil.logDataObject(LOGGER, CLASSNAME, "getDataObject(byte[], HashMap)", //$NON-NLS-1$
                                  responseObj);
        return responseObj;
    }


    /**
     * Converts fixed width string to a data object. It uses a hashmap of
     * configuration properties. The default values for these properties
     * are set in the constructor.
     * If the config is null, the default values are used for parsing the
     * stream. The default values can be overridden by passing them in the
     * config hashmap.
     *
     * @param config Map of configuration properties
     * @return Dataobject that corresponds to the byte stream
     * @param serializedData 
     */
    public DataObject getDataObject(String serializedData,
                                    HashMap config)
	throws FixedWidthHandlerException
    {
        logEntry("getDataObject(String, HashMap)");  //$NON-NLS-1$
        setupConfig(config);

        //input String is not modified by getNextToken calls
        //store reference directly
        dataObjectString = serializedData;
        dataObjectStringLength = dataObjectString.length();
        DataObject resultObj = getDataObjectFromString(null);
        
        HandlerUtil.logDataObject(LOGGER, CLASSNAME, "getDataObject(String, HashMap)", resultObj);  //$NON-NLS-1$        
        logExit("getDataObject(String, HashMap)");  //$NON-NLS-1$
        return resultObj;
    }


    /**
     * Populates a given DataObject with data extracted from String object. In
     * this case it is up to the caller to create the BO. Allows configuration
     * options to be set by passing in a map of configuration properties
     *
     * @return DataObject
     * @param config
     * @param serializedData
     * @param theObj 
     */
    public DataObject getDataObject(String serializedData,
                              DataObject theObj,
                              HashMap config)
		throws FixedWidthHandlerException
    {
        logEntry("getDataObject(String, DataObject, HashMap)");  //$NON-NLS-1$
        
        setupConfig(config);
        dataObjectString = serializedData;
        dataObjectStringLength = serializedData.length();
        DataObject resultObj = getDataObjectFromString(theObj);
        
        HandlerUtil.logDataObject(LOGGER, CLASSNAME, "getDataObject(String, HashMap)", resultObj);  //$NON-NLS-1$
        logExit("getDataObject(String, DataObject, HashMap)");  //$NON-NLS-1$
        return resultObj;
    }


    /**
     * Creates a DataObject if the value passed in is null. Calls
     * parseAttributeList to do the parsing of the String and populate the
     * attribute values. Determines what type of data object to create either
     * based on the parameter value or on the first token in the data. The
     * paramter value must match the first token in the data, otherwise an error
     * is generated.
     *
     * @return DataObject
     * @param dataObject 
     */
    protected DataObject getDataObjectFromString(DataObject dataObject)
    	throws FixedWidthHandlerException
    {
        logEntry("getDataObject(DataObject)");  //$NON-NLS-1$
        
        String dataObjectName = null;
        if (dataObject != null) {
            dataObjectName = dataObject.getType().getName();
        }
        String streamDOName = this.getNextToken(dataObjectNameSize, true);
        if ((dataObjectName != null) && (streamDOName.compareTo(dataObjectName) != 0))
        {
            String message = "Name from input source " + streamDOName + " does not match with " +  //$NON-NLS-2$  //$NON-NLS-1$
                    "the one from the stream " + dataObjectName;  //$NON-NLS-1$
            LOGGER.logp(Level.SEVERE, CLASSNAME, "getDataObject(byte[], HashMap)", message);  //$NON-NLS-1$
            throw new FixedWidthHandlerException(message);
        }
        if (dataObject == null)
        {
            dataObject = BOFACTORY.create(targetNamespace, streamDOName);
            LOGGER.logp(Level.FINER, CLASSNAME, "getDataObjectFromString(String, HashMap)", //$NON-NLS-1$
                        "Created a DataObject: \"{0}\"", targetNamespace+"//"+dataObjectName);  //$NON-NLS-1$            
        }
        parseAllProperties(dataObject);
        
        HandlerUtil.logDataObject(LOGGER, CLASSNAME, "getDataObject(String, HashMap)", dataObject);  //$NON-NLS-1$
        logExit("getDataObject(String, DataObject, HashMap)");  //$NON-NLS-1$
        
        return dataObject;
    }


    /**
     * Converts a dataobject to a fixed width string. It uses a hashmap of
     * configuration properties. The default values for these properties
     * are set in the constructor.
     * If the config is null, the default values are used for serializing the
     * dataobject to string. The default values can be overridden by passing them in the
     * config hashmap.
     *
     * @param config Map of configuration properties
     * @return Fixed width string that corresponds to the dataobject
     * @param dataObject 
     */
    public String getStringFromDataObject(DataObject dataObject,
                                          HashMap config)
		throws FixedWidthHandlerException
    {
        logEntry("getStringFromDataObject(DataObject, HashMap)");  //$NON-NLS-1$
        HandlerUtil.logDataObject(LOGGER, CLASSNAME, "getStringFromDataObject(DataObject, HashMap)", dataObject);  //$NON-NLS-1$
        HandlerUtil.logConfig(LOGGER, CLASSNAME, "getStringFromDataObject(DataObject, HashMap)", config);  //$NON-NLS-1$

        setupConfig(config);       
        //Add the business object name
        Type type = dataObject.getType();
        String name = ExtendedMetaData.INSTANCE.getName(((EType) type).getEClassifier());
        StringBuffer dataObjectStrBuf = new StringBuffer();
        this.setSimpleToken(dataObjectNameSize, name, dataObjectStrBuf);
        serializeAllProperties(dataObject, dataObjectStrBuf);
        String dataObjectStr = dataObjectStrBuf.toString();
        
        LOGGER.logp(Level.FINER, CLASSNAME, "getStringFromDataObject(DataObject dataObject, HashMap map)", //$NON-NLS-1$
                    "Created the Stream : \"{0}\"", dataObjectStr);  //$NON-NLS-1$
        logExit("getStringFromDataObject(DataObject, HashMap)");  //$NON-NLS-1$
        return dataObjectStr;
    }

    /**
     * Converts a dataobject to a fixed width byte stream. It uses a hashmap of
     * configuration properties. The default values for these properties
     * are set in the constructor.
     * If the config is null, the default values are used for serializing the
     * dataobject to byte stream. The default values can be overridden by passing them in the
     * config hashmap.
     *
     * @param config Map of configuration properties
     * @return Delimited byte stream that corresponds to the dataobject
     * @param dataObject 
     */
    public byte[] getBytesFromDataObject(DataObject dataObject,
                                         HashMap config)
		throws FixedWidthHandlerException
    {
        logEntry("getBytesFromDataObject(DataObject, HashMap)");  //$NON-NLS-1$
        HandlerUtil.logDataObject(LOGGER, CLASSNAME, "getStringFromDataObject(DataObject, HashMap)", dataObject);  //$NON-NLS-1$
        HandlerUtil.logConfig(LOGGER, CLASSNAME, "getStringFromDataObject(DataObject, HashMap)", config);  //$NON-NLS-1$

        setupConfig(config);
        String dataObjectStr = getStringFromDataObject(dataObject, config);
      
        LOGGER.logp(Level.FINER, CLASSNAME, "getStringFromDataObject(DataObject dataObject, HashMap map)", //$NON-NLS-1$
                    "Created the Stream, the Stream is : \"{0}\"", dataObjectStr);  //$NON-NLS-1$

        try {
            if (encoding != null) {
                logExit("getBytesFromDataObject(DataObject, HashMap)");  //$NON-NLS-1$
                return dataObjectStr.getBytes(encoding);
            }
            logExit("getBytesFromDataObject(DataObject, HashMap)");  //$NON-NLS-1$
            return dataObjectStr.getBytes();

        } catch (UnsupportedEncodingException e) {
            String message = "The encoding type " + encoding + " is not supported";  //$NON-NLS-2$  //$NON-NLS-1$
            LOGGER.logp(Level.SEVERE, CLASSNAME, "getDataObject(byte[], HashMap)", message);  //$NON-NLS-1$
            throw new FixedWidthHandlerException(message, e);
        }
    }


    /**
     * Parses dataObjectString to populate the attributes for the DataObject.
     * 
     * @param dataObject DataObject that is created from the incoming stream of data.
     */
    protected void parseAllProperties(DataObject dataObject)
		throws FixedWidthHandlerException
    {
        logEntry("parseAllProperties(DataObject)");  //$NON-NLS-1$
        if (dataObject == null)
        {
            String message = "The Input Stream has no tokens in it or the DataObject is null";  //$NON-NLS-1$
            LOGGER.logp(Level.SEVERE, CLASSNAME, "parseAttributeList(DataObject dataObject)", message);  //$NON-NLS-1$
            throw new FixedWidthHandlerException("parseAllProperties received null DataObject");  //$NON-NLS-1$
        }

        Iterator propertyIterator = dataObject.getType().getProperties().iterator();
        while (propertyIterator.hasNext()) {
            Property property = (Property) propertyIterator.next();
            String propertyName = property.getName();

            //simple type non list
            if (!property.isContainment() && !property.isMany()) {
                parseSimpleProperty(dataObject, property, propertyName);
            }

            //simple type list
            if (!property.isContainment() && property.isMany()) {
                parseSimpleTypeList(dataObject, property, propertyName);
            }

            //complex type non list
            if (property.isContainment() && !property.isMany()) {
                parseChildDataObject(dataObject, propertyName);
                continue;
            }

            //complex type property
            if (property.isContainment() && property.isMany()) {                
                parseChildDataObjectList(dataObject, propertyName);
                continue;
            }
        }
        logExit("parseAllProperties(DataObject)");  //$NON-NLS-1$
    }


    /**
     * Populates a child data object list based on token values in the
     * fixedwidth string. Expects all objects in the list to be of the same
     * type.
     * 
     * @param parentDataObject Parent data object.
     * @param propertyName Name of the property that corresponds to the child data object
     */
    protected void parseChildDataObjectList(DataObject parentDataObject,
                                            String propertyName)
		throws FixedWidthHandlerException
    {
        logEntry("parseChildDataObject(DataObject, String)");  //$NON-NLS-1$
        LOGGER.logp(Level.FINE, CLASSNAME, "parseChildDataObject(DataObject, String)",  //$NON-NLS-1$
                    "Parsing child data object \"{0}\"", propertyName);  //$NON-NLS-1$
        
        String propertyValue = this.getNextToken(dataObjectCountSize, true);
        if (propertyValue.equals(unset.toString())) {
            return;
        }

        int objectCount = Integer.parseInt(propertyValue);
        if (objectCount == 0) {
            return;
        }

        if (objectCount < 0) {
            String message = "The list of child data objects " + propertyName + " has less than 0 elements";  //$NON-NLS-2$  //$NON-NLS-1$
            LOGGER.logp(Level.SEVERE, CLASSNAME, "parseAttributeList(DataObject dataObject)", message);  //$NON-NLS-1$              
            throw new FixedWidthHandlerException("The object count cannot be less than 0");  //$NON-NLS-1$
        }

        // For each instance of the child object, parse the
        //  attribute list, and then add the object container
        //  to the parent.
        for (int index = 0; index < objectCount; index++) {
            String childObjectName = getNextToken(dataObjectNameSize, true);
            if (!childObjectName.equals(propertyName)) {
                String message = "The stream contains a BO " + childObjectName + " that is not in the actual Business Object" + propertyName;  //$NON-NLS-2$  //$NON-NLS-1$
                LOGGER.logp(Level.SEVERE, CLASSNAME, "parseAttributeList(DataObject dataObject)", message);  //$NON-NLS-1$              
                throw new FixedWidthHandlerException(message);                                                     
            }            
            DataObject childDataObject = parentDataObject.createDataObject(propertyName);
            parseAllProperties(childDataObject);
        }
        logExit("parseChildDataObject(DataObject, String)");  //$NON-NLS-1$
    }


    /**
     * Populates a single cardinatlity child business object with the token
     * values found in the fixed width string.
     * 
     * @param parentDataObject Parent data object.
     * @param propertyName Name of the property for the child business object.
     */
    protected void parseChildDataObject(DataObject parentDataObject,
                                        String propertyName)
		throws FixedWidthHandlerException
    {
        logEntry("parseChildDataObject(DataObject, String)");  //$NON-NLS-1$       
        LOGGER.logp(Level.FINE, CLASSNAME, "parseChildDataObject(DataObject, String)",  //$NON-NLS-1$
                    "Parsing child data object named \"{0}\"", propertyName);  //$NON-NLS-1$
        
        String propertyValue = this.getNextToken(dataObjectNameSize, true);
        if (propertyValue.equals(unset.toString())) {
            return;
        }
        
        if (!propertyValue.equals(propertyName)) {
            String message = "The stream contains a BO " + propertyValue + " that is not in the business object" + propertyName;  //$NON-NLS-2$  //$NON-NLS-1$
            LOGGER.logp(Level.SEVERE, CLASSNAME, "parseAttributeList(DataObject dataObject)", message);  //$NON-NLS-1$                      
            throw new FixedWidthHandlerException(message);
        }
        // Check the object count token
        // If it does not match "1", assume that the child object should
        //  be null
        // The delimited string contains a single instance of the
        //  child object
        DataObject childDataObject = parentDataObject.createDataObject(propertyName);
        parseAllProperties(childDataObject);
        
        logExit("parseChildDataObject(DataObject, String)");  //$NON-NLS-1$    
    }


    /**
     * Populates a list of simple types with the token values found in the fixed
     * width string. 
     * 
     * @param dataObject Parent data object. 
     * @param property Property corresponding to the simple type list 
     * @param propertyName Type of the simple types in the list.
     */
    protected void parseSimpleTypeList(DataObject dataObject,
                                       Property property,
                                       String propertyName)
		throws FixedWidthHandlerException
    {
        logEntry("parseSimpleTypeList(DataObject, Property, String)");  //$NON-NLS-1$
        LOGGER.logp(Level.FINE, CLASSNAME, "parseSimpleTypeList(DataObject, Property, String)",  //$NON-NLS-1$
                    "Parsing simple type list \"{0}\"", propertyName);  //$NON-NLS-1$
        
        String propertyValue = this.getNextToken(dataObjectCountSize, true);
        
        //propertyValue is unset, do nothing
        if (propertyValue.equals(unset)) {
            return;
        }

        //the list size token in the string is 0, there are no elements, do nothing
        int listSize = Integer.parseInt(propertyValue);
        if (listSize == 0) {
            return;
        }

        if (listSize < 0) {
            throw new FixedWidthHandlerException("The list size is less than 0. This is an illegal value");  //$NON-NLS-1$
        }

        List list = dataObject.getList(propertyName);
        int propertyWidth = getMaxLength((EProperty) property);
        for (int i = 0; i < listSize; i++) {
            //Get the next token based on the MaxLength of the attribute
            propertyValue = this.getNextToken(propertyWidth, false);

            if (propertyValue == null) {
                String message = "There is a null value in the list " + propertyName;  //$NON-NLS-1$
                LOGGER.logp(Level.SEVERE, CLASSNAME, "parseAttributeList(DataObject dataObject)", message);  //$NON-NLS-1$                      
                throw new FixedWidthHandlerException(message);
            }
            if (propertyValue.equals(unset) || "".equals(propertyValue)) {  //$NON-NLS-1$
                String message = "Encountered an unset value in the list " + propertyName;  //$NON-NLS-1$
                LOGGER.logp(Level.SEVERE, CLASSNAME, "parseAttributeList(DataObject dataObject)", message);  //$NON-NLS-1$                                      
                throw new FixedWidthHandlerException(message);
            }
            if (propertyValue.equals(blank.toString()) || " ".equals(propertyValue)) {  //$NON-NLS-1$
                String message = "Encountered a blank value in the list " + propertyName;  //$NON-NLS-1$
                LOGGER.logp(Level.SEVERE, CLASSNAME, "parseAttributeList(DataObject dataObject)", message);  //$NON-NLS-1$                      
                throw new FixedWidthHandlerException("Encountered a blank in the list");  //$NON-NLS-1$
            }
            EStructuralFeature eFeature = ((EProperty) property).getEStructuralFeature();
            EDataType dataType = (EDataType) eFeature.getEType();
            //convert the string token to the type that the list contains and add to list
            Object value = EcoreUtil.createFromString(dataType, propertyValue);
            list.add(value);
        }
        
        logExit("parseSimpleTypeList(DataObject, Property, String)");  //$NON-NLS-1$
    }

    /**
     * Reads the token value and sets the value for a simple type property  
     * @param dataObject Parent data object. 
     * @param property Property corresponding to the simple type
     * @param propertyName name of the property that corresponds to the simple type
     */
    protected void parseSimpleProperty(DataObject dataObject,
                                       Property property,
                                      String propertyName)
		throws FixedWidthHandlerException
    {
        logEntry("parseSimpleProperty(DataObject, Property, String");  //$NON-NLS-1$
        LOGGER.logp(Level.FINE, CLASSNAME, "parseSimpleTypeList(DataObject, Property, String)",  //$NON-NLS-1$
                    "Parsing simple type property : \"{0}\"", propertyName);  //$NON-NLS-1$
        
        // For simple (String) attribute, we check to see if the token is either
        //  null, or matches the configured unset or Blank values.
        //  Otherwise, set the attribute value with the token value
        int boListSize = getMaxLength((EProperty) property);
        String propertyValue = this.getNextToken(boListSize, false);

        if (propertyValue == null)
        {                    
            return;
        }
        if (propertyValue.equals(unset.toString()) || "".equals(propertyValue)) {  //$NON-NLS-1$
            dataObject.unset(propertyName);
            return;
        }
        if (propertyValue.equals(blank.toString()) || " ".equals(propertyValue)) {  //$NON-NLS-1$
            dataObject.set(propertyName, "");  //$NON-NLS-1$
            return;
        }
        dataObject.setString(propertyName, propertyValue);
        logExit("parseSimpleProperty(DataObject, Property, String");  //$NON-NLS-1$
    }

    /**
     * Get the max length for the property. This is set in the schema definition of the 
     * dataobject
     * @param eProperty Property in the dataobject for which max length is to be determined 
     * @return max length of the given property
     */
    protected int getMaxLength(EProperty eProperty)
    {
        EDataType dataType = (EDataType) eProperty.getEStructuralFeature().getEType();
        int maxLength = ExtendedMetaData.INSTANCE.getMaxLengthFacet(dataType);
        return maxLength;
    }

    /**
     * Serialize the simple type property value as a string
     * @param dataObject The property value to be serialized is part of this dataobject
     * @param property The value of this property is serialized
     * @param dataObjectStrBuf The string buffer that contains the fixed width string
     */
    protected void serializeSimpleProperty(DataObject dataObject,
                                           Property property,
                                           StringBuffer dataObjectStrBuf)
		throws FixedWidthHandlerException
    {
        logEntry("serializeSimpleProperty(DataObject, Property, StringBuffer)");  //$NON-NLS-1$
        
        String propertyName = property.getName();
        LOGGER.logp(Level.FINE, CLASSNAME, "serializeSimpleProperty(DataObject, Property, StringBuffer)",  //$NON-NLS-1$
                    "Serializing simple type property \"{0}\"", propertyName);  //$NON-NLS-1$
        
        String propertyValue = null;
        if (dataObject.isSet(propertyName)) {
            propertyValue = dataObject.getString(propertyName);
        }

        int maxLength = getMaxLength((EProperty) property);
        setSimpleToken(maxLength, propertyValue, dataObjectStrBuf);
        
        logExit("serializeSimpleProperty(DataObject, Property, StringBuffer)");  //$NON-NLS-1$
    }

    /**
     * Serialize the property values of child data object to a fixed width string
     * 
     * @param dataObject The property value to be serialized is part of this dataobject
     * @param property The value of this property is serialized
     * @param dataObjectStrBuf The string buffer that contains the fixed width string
     */
    protected void serializeChildDataObject(DataObject dataObject,
                                            Property property,
                                            StringBuffer dataObjectStrBuf)
		throws FixedWidthHandlerException
    {
        logEntry("serializeChildDataObject(DataObject, Property, StringBuffer)");  //$NON-NLS-1$
        
        String propertyName = property.getName();
        LOGGER.logp(Level.FINE, CLASSNAME, "serializeChildDataObject(DataObject, Property, StringBuffer)",  //$NON-NLS-1$
                    "Serializing complex type property \"{0}\"", propertyName);  //$NON-NLS-1$
        
        DataObject childDataObject = dataObject.getDataObject(propertyName);

        //add unset to the fixed width string
        if (childDataObject == null) {
            //TODO:trace
            this.setSimpleToken(fieldWidth, null, dataObjectStrBuf);
            return;
        }
        //TODO:trace
        //add the attributes in the child data object
        setSimpleToken(fieldWidth, propertyName, dataObjectStrBuf);
        serializeAllProperties(childDataObject, dataObjectStrBuf);
        
        logExit("serializeChildDataObject(DataObject, Property, StringBuffer)");  //$NON-NLS-1$
    }

    /**
     * Convert the value of any type to a string type so it can be tokenized to the stream
     *
     * @param property Property whose value is to be converted to string
     * @param objValue value to be converted to string
     * @return String
     */
    protected String convertToString(Property property,
                                     Object objValue)
    {
        EDataType dataType = (EDataType) ((EProperty) property).getEStructuralFeature().getEType();
        String value = EcoreUtil.convertToString(dataType, objValue);
        return value;
    }

    /**
     * First serialize the size of the list, next serialize the name of the property
     * corresponding to the child data object from the list and serialize its
     * property values to a fixed width string.
     * 
     * @param dataObject The property value to be serialized is part of this dataobject
     * @param property The value of this property is serialized
     * @param dataObjectStrBuf The string buffer that contains the fixed width string
     */
    protected void serializeChildDataObjectList(DataObject dataObject,
                                                Property property,
                                                StringBuffer dataObjectStrBuf)
		throws FixedWidthHandlerException
    {
        logEntry("serializeChildDataObjectList(DataObject, Property, StringBuffer)");  //$NON-NLS-1$
        String propertyName = property.getName();
        LOGGER.logp(Level.FINE, CLASSNAME, "serializeChildDataObjectList(DataObject, Property, StringBuffer)",  //$NON-NLS-1$
                    "Serializing complex type list property \"{0}\"", propertyName);  //$NON-NLS-1$
        
        List list = dataObject.getList(propertyName);

        //list is null or empty, so add unset and return
        if (list == null || list.isEmpty()) {
            this.setSimpleToken(fieldWidth, null, dataObjectStrBuf);
            return;
        }

        // Add list size to the fixed width String
        int listSize = list.size();
        this.setSimpleToken(dataObjectCountSize, Integer.toString(listSize), dataObjectStrBuf);

        //a list of complex types
        for (int listCounter = 0; listCounter < listSize; ++listCounter) {
            setSimpleToken(fieldWidth, propertyName, dataObjectStrBuf);
            serializeAllProperties((DataObject) list.get(listCounter), dataObjectStrBuf);
        }
        logExit("serializeChildDataObjectList(DataObject, Property, StringBuffer)");  //$NON-NLS-1$
    }

    /**
     * Serialize out the size of the list and then the values in the list.
     *
     * @param dataObject The DataObject that contains the list of simple type
     * @param property The property that corresponds to the list of simple type
     * @param dataObjectStrBuf 
     */
    protected void serializeSimpleTypeList(DataObject dataObject,
                                           Property property,
                                           StringBuffer dataObjectStrBuf)
		throws FixedWidthHandlerException
    {
        logEntry("serializeSimpleTypeList(DataObject, Property, StringBuffer)");  //$NON-NLS-1$
        
        String propertyName = property.getName();
        
        LOGGER.logp(Level.FINE, CLASSNAME, "serializeSimpleTypeList(DataObject, Property, StringBuffer)",  //$NON-NLS-1$
                    "Serializing simple type list property \"{0}\"", propertyName);  //$NON-NLS-1$
        
        List list = dataObject.getList(propertyName);

        //list is null or empty, so add unset and return
        if (list == null || list.isEmpty()) {
            this.setSimpleToken(fieldWidth, null, dataObjectStrBuf);
            return;
        }

        // Add list size to the fixed width String
        int listSize = list.size();
        this.setSimpleToken(dataObjectCountSize, Integer.toString(listSize), dataObjectStrBuf);

        Iterator listIterator = list.iterator();
        while (listIterator.hasNext()) {
            Object listValue = listIterator.next();
            String propertyValue = convertToString(property, listValue);
            int maxLength = getMaxLength((EProperty) property);
            this.setSimpleToken(maxLength, propertyValue, dataObjectStrBuf);
        }
        
        logExit("serializeSimpleTypeList(DataObject, Property, StringBuffer)");  //$NON-NLS-1$
    }


    /**
     * Loops through the properties in a DataObject and add the attribute values
     * to the dataObjectStringBuffer.
     *
     * @param dataObject DataObject which contains the data to be added to the fixed
     * width String.
     * @param dataObjectStrBuf 
     */
    protected void serializeAllProperties(DataObject dataObject,
                                          StringBuffer dataObjectStrBuf)
		throws FixedWidthHandlerException
    {
        logEntry("serializeAllProperties(DataObject, StringBuffer)");  //$NON-NLS-1$
        
        Type type = dataObject.getType();
        //Add properties
        Iterator propertyIterator = type.getProperties().iterator();
        while (propertyIterator.hasNext()) {
            Property property = (Property) propertyIterator.next();
            boolean isContainment = property.isContainment();
            boolean isMany = property.isMany();

            //non list, simple type property
            if (isContainment == false && isMany == false) {
                serializeSimpleProperty(dataObject, property, dataObjectStrBuf);
                continue;
            }

            //list of simple types
            if (isContainment == false && isMany) {
                serializeSimpleTypeList(dataObject, property, dataObjectStrBuf);
                continue;
            }

            //non list, complex type
            if (isContainment && isMany == false) {
                serializeChildDataObject(dataObject, property, dataObjectStrBuf);
                continue;
            }

            // list of complex types
            if (isContainment && isMany) {
                serializeChildDataObjectList(dataObject, property, dataObjectStrBuf);
                continue;
            }
        }
        logExit("serializeAllProperties(DataObject, StringBuffer)");  //$NON-NLS-1$
    }

    /**
     * Parses the next token from dataObjectString based on the length of the
     * token. Also trims the field of whitespace.
     * 
     * @param tokenLength The length (number of characters) of the next token.
     * @param trimBoth If 'true' passed in, then the token is trimmed on both sides.
     *            If 'false' is passed, it is only trimmed according to the
     *            option set.
     * @return String value that is the next token that appers in the dataObjectString.
     */
    protected String getNextToken(int tokenLength,
                                  boolean trimBoth)
		throws FixedWidthHandlerException
    {
        logEntry("getNextToken(int, boolean)");  //$NON-NLS-1$
        
        if (startIndex >= dataObjectStringLength)
        {
            String message = "Reached end of BO while expecting more token";  //$NON-NLS-1$
            LOGGER.logp(Level.SEVERE, CLASSNAME, "parseAttributeList(DataObject dataObject)", message);  //$NON-NLS-1$                              
            throw new FixedWidthHandlerException(message);
        }
        String resultToken = null;
        if (tokenLength < 1 && fieldWidth == 0)
        {
            String message = "The token cannot be extracted from the stream since the width of the token is unknown";  //$NON-NLS-1$
            LOGGER.logp(Level.SEVERE, CLASSNAME, "parseAttributeList(DataObject dataObject)", message);  //$NON-NLS-1$                                       
            throw new FixedWidthHandlerException(message);
        }
        if (tokenLength < 1) {
            tokenLength = fieldWidth;
        }

        if (tokenLength > 0) //else no sense to do anything
        {
            int leftIndex = startIndex;
            int rightIndex = startIndex + tokenLength;

            if (rightIndex > dataObjectStringLength) {
                rightIndex = dataObjectStringLength;
            } // take care of the last
                                                     // token

            //trim according to configuration
            if (trimBoth || alignment.equalsIgnoreCase(FixedWidthDataHandler.ALIGNMENTBOTH)) {
                leftIndex = this.trimLeft(leftIndex, rightIndex);
                rightIndex = this.trimRight(rightIndex, leftIndex);
            }
            if (alignment.equalsIgnoreCase(FixedWidthDataHandler.ALIGNMENTLEFT)) {
                rightIndex = this.trimRight(rightIndex, leftIndex);
            }
            if (alignment.equalsIgnoreCase(FixedWidthDataHandler.ALIGNMENTRIGHT)) {
                leftIndex = this.trimLeft(leftIndex, rightIndex);
            }

            resultToken = dataObjectString.substring(leftIndex, rightIndex);
            startIndex += tokenLength;
        }
        logExit("getNextToken(int, boolean)");  //$NON-NLS-1$
        return resultToken;
    }

    /**
     * Compute start of token data in dataObjectString by checking for pad
     * characters starting on the left of the cell and moving right.
     *
     * @param beginIndex Start index of the cell of interest in dataObjectString
     * @param endIndex End index of the cell of interest in dataObjectString
     * @return Starting index of data in cell of interest.     
     */
    protected int trimLeft(int beginIndex,
                           int endIndex)
    {
        int leftIndex = beginIndex;
        while (padCharacter == dataObjectString.charAt(leftIndex) && leftIndex < dataObjectStringLength
               && leftIndex < endIndex) {
            leftIndex++;
        }
        return leftIndex;
    }


    /**
     * Compute start of token data dataObjectString by checking for pad
     * characters starting on the right of the cell and moving left.
     *
     * @return Ending index of data in cell of interest.
     * @param endIndex
     * @param beginIndex 
     */
    protected int trimRight(int beginIndex,
                            int endIndex)
    {
        int rightIndex = beginIndex - 1;
        while (padCharacter == dataObjectString.charAt(rightIndex) && 0 < rightIndex && endIndex < rightIndex) {
            rightIndex--;
        }
        rightIndex++; // Compensates for the last checked character

        return rightIndex;
    }

    /**
     * Pad the token with the pad characters and then append the token in the 
     * dataObjectStrBuf
     * 
     * @param cellSize Number of characters to add to this String for this token.
     * @param tokenValue Characters to be added to the String for this token.
     * @param dataObjectStrBuf The String buffer that contains the fixed width stream.
     */
    protected void setToken(int cellSize,
                            StringBuffer tokenValue,
                            StringBuffer dataObjectStrBuf)
		throws FixedWidthHandlerException
    {
        logEntry("setToken(int, StringBuffer, StringBuffer");  //$NON-NLS-1$
        int tokenLength = tokenValue.length();
        String value = tokenValue.toString();

        if (!truncate && tokenLength > cellSize) {
            throw new FixedWidthHandlerException(" The size of token " + tokenValue + " is greater than the cellSize"  //$NON-NLS-1$  //$NON-NLS-2$
                                                 + cellSize + " and trucation is not allowed");  //$NON-NLS-1$
        }
        int padNumber = cellSize - tokenLength;
        if (padNumber < 0 && truncate) {
            value = tokenValue.substring(0, cellSize);
        }
        if (padNumber > 0) {
            if (alignment.equalsIgnoreCase(FixedWidthDataHandler.ALIGNMENTLEFT)
                || alignment.equalsIgnoreCase(FixedWidthDataHandler.ALIGNMENTBOTH)) {
                value = padRight(tokenValue, padNumber);
            
            } else {
                value = padLeft(tokenValue, padNumber);
            }
        }
        dataObjectStrBuf.append(value);
        logExit("setToken(int, StringBuffer, StringBuffer");  //$NON-NLS-1$
    }


    /**
     * Adds a token value with correct pad char if needed to the data String
     * being created (dataObjectStringBuffer). Also contains the logic to check
     * for the escape and delimiter character in the token value passed in if
     * needed.
     * 
     * @param cellSize Number of characters to add to this String for this token.
     * @param tokenValue Characters to be added to the String for this token.
     * @param dataObjectStrBuf The String buffer that contains the fixed width stream.
     */
    protected void setSimpleToken(int cellSize,
                                  String tokenValue,
                                  StringBuffer dataObjectStrBuf)
		throws FixedWidthHandlerException
    {
        logEntry("setToken(int, String, StringBuffer");  //$NON-NLS-1$
        if (cellSize <= 0) {
            cellSize = fieldWidth;
        }

        // Check the token value to see if it is null
        if (tokenValue == null) {
            setToken(cellSize, unset, dataObjectStrBuf);
            return;
        }

        if ("".equals(tokenValue)) {  //$NON-NLS-1$
            setToken(cellSize, blank, dataObjectStrBuf);
            return;
        }

        setToken(cellSize, new StringBuffer(tokenValue), dataObjectStrBuf);

        logExit("setToken(int, String, StringBuffer");  //$NON-NLS-1$
    }


    /**
     * Append indicated number of instances of the pad character to the left of
     * the data for the cell of interest.
     *
     * @param stringBuffer StringBuffer to which the characters are added. Value changed
     *            by this method.
     * @param padCharNum Number of characters which to add to the left of the token
     *            data contained in pvStringBuffer.
     * @return String
     */
    protected String padLeft(StringBuffer stringBuffer,
                             int padCharNum)
    {
        for (int charIndex = 0; charIndex < padCharNum; charIndex++) {
            stringBuffer.insert(0, padCharacter);
        }
        return stringBuffer.toString();
    }


    /**
     * Append indicated number of instances of the pad character to the right of
     * the data for the cell of interest.
     *
     * @param stringBuffer StringBuffer to which the characters are added. Value changed
     *            by this method.
     * @param padCharNum Number of characters which to add to the right of the token
     *            data contained in pvStringBuffer.
     * @return String
     */
    protected String padRight(StringBuffer stringBuffer,
                              int padCharNum)
    {
        for (int charIndex = 0; charIndex < padCharNum; charIndex++) {
            stringBuffer.append(padCharacter);
        }
        return stringBuffer.toString();
    }

    private void logEntry(String message)
    {
        if (LOGGER.isLoggable(Level.INFO)) {
            LOGGER.entering(CLASSNAME, message);
        }
    }


    private void logExit(String message)
    {
        if (LOGGER.isLoggable(Level.INFO)) {
            LOGGER.exiting(CLASSNAME, message);
        }
    }
}
